/**
 * @file
 * @brief Implementation of system events.
 */
#ifndef STA_RTOS_SYSTEM_EVENTS_HPP
#define STA_RTOS_SYSTEM_EVENTS_HPP


/**
 * @defgroup STA_RTOS_SysEvent System Events
 * @ingroup STA_RTOS_API
 * @brief System events.
 *
 * Check @ref STA_RTOS_BuildConfig for configuration options.
 */


#include <cstdint>


// System event flags
//

/**
 * @brief Startup system event flag.
 *
 * @ingroup STA_RTOS_SysEvent
 */
#define STA_RTOS_SYSTEM_EVENTS_STARTUP 0x100000U


namespace sta
{
	namespace rtos
	{
		/**
		 * @brief Initialize system events.
		 */
		void initSystemEvents();


		/**
		 * @brief Signal system events.
		 *
		 * @param flags     System event flags
		 *
		 * @ingroup STA_RTOS_SysEvent
		 */
		void signalSystemEvents(uint32_t flags);

		/**
		 * @brief Wait for system events.
		 *
		 * @param flags     System event flags
		 * @param options   osFlagsWaitAll or osFlagsWaitAny (osFlagsNoClear always set)
		 * @param timeout   Wait timeout (0 = instant, osWaitForever = infinite)
		 *
		 * @ingroup STA_RTOS_SysEvent
		 */
		void waitForSystemEvents(uint32_t flags, uint32_t options, uint32_t timeout);


		/**
		 * @brief Signal startup system event.
		 *
		 * @ingroup STA_RTOS_SysEvent
		 */
		void signalStartupEvent();
	
		/**
		 * @brief Wait for startup system event.
		 *
		 * Blocking while waiting
		 *
		 * @ingroup STA_RTOS_SysEvent
		 */
		void waitForStartupEvent();
	} // namespace rtos
} // namespace sta


#endif // STA_RTOS_SYSTEM_EVENTS_HPP
